// SPDX-License-Identifier: Apache-2.0

//! Modules for interfacing with SEV firmware.
//! Rust-friendly API wrappers to communicate with the FFI functions.
pub mod guest;
pub mod host;

pub(crate) mod linux;
